package RPM::Source::Transform;

use strict;
use warnings;

use Carp;
use Getopt::Long;
use RPM::Source::Editor;
use RPM::Source::Abstract::PriorityCollection;
use RPM::Source::TransformContainer;
use RPM::Source::Shared::Resource::Debug;
use RPM::Source::Transformation::DefaultPlayer;
use RPM::Source::Transformation::MessageBus;

# moved to subclasses that really use it
#use RPM::Source::Transformation::RPMEditorProvider;


our $VERSION=0.016;

our $debug_container;

my $help;

our $verbose=1;

our @PROTECTED_OPTIONS=(
    'help'  => \$help,
    'v|verbose+'  => \$verbose,
    'q|quiet'   => sub { $verbose = 0 },
);
# not used
# our @EXT_OPTS;

sub usage {
    my ($self)=@_;
    print "General options:
	-h, --help
	-v, --verbose (can be specified multiple times)
	-q, --quiet
";
    &longopt_usage();
}

sub longopt_usage {
    RPM::Source::TransformContainer->longopt_usage;
}

sub new {
    my $class = shift;
    my $self = {
	@_
    };
    bless $self, $class;
    my $not_embedded=$RPM::Source::TransformContainer::INPUT_HANDLER && ! $self->{-embedded};
    if ($not_embedded) {
	$self->get_options();
    }
    $self->process_options();

    RPM::Source::TransformContainer->debug_container if $debug_container;

    $self->{-resourceobj}=RPM::Source::Shared::ResourceContainer->new();
    $self->_init_transformations();
    if ($not_embedded) {
	$RPM::Source::TransformContainer::INPUT_HANDLER->process_args();
	if (not $RPM::Source::TransformContainer::INPUT_HANDLER->verify_args()) {
	    print STDERR "ERROR: invalid or missing input arguments.\n";
	    exit(4);
	}
    }
    return $self;
}


sub get_longopt {
    my ($self)=@_;
    return RPM::Source::TransformContainer->get_longopt;
}

# can be used both as class and object method (for Embedded usage)
# in the past was used as class method in octave-package-builder
sub get_options {
    my ($self)=@_;
    my $result = GetOptions ($self->get_longopt,@PROTECTED_OPTIONS);
    if (not $result) {
	print "run $0 --help for details.\n";
	exit 1;
    } elsif ($help) {
	$self->usage();
	exit 0;
    }
}

sub before_process_options {
    RPM::Source::TransformContainer->before_process_options;
}

sub process_options {
    RPM::Source::TransformContainer->process_options;
}

sub mk_opt {
#    my ($self)=@_;
    my %opt;
    tie %opt, 'RPM::Source::Transformation::MessageBus';
    # @RESOURCE_CLASSES don't register_messages
    map {$_->register_messages(\%opt)} 
        @RPM::Source::TransformContainer::SUPPORT_CLASSES, 
        @RPM::Source::TransformContainer::FACTORY_CLASSES;
    return \%opt;
}

sub _init_transformations {
    my ($self)=@_;
    my $resourceobj=$self->{-resourceobj};
    map {$_->init_factory($resourceobj)} @RPM::Source::TransformContainer::FACTORY_CLASSES;
}

sub transform_once {
    my ($self,$spec,$parent,$opt)=@_;

    my $resourceobj=$self->{-resourceobj};
    map {$_->prepare_resource($spec,$parent,$opt,$resourceobj)} @RPM::Source::TransformContainer::RESOURCE_CLASSES;

    my $actions=RPM::Source::Abstract::PriorityCollection->new();
    $actions->add(map {$_->transformations($opt,$resourceobj)} @RPM::Source::TransformContainer::FACTORY_CLASSES);

    if ($RPM::Source::TransformContainer::PLAYER->apply_collection($spec,$parent,$opt,$resourceobj,$actions) == RPM::Source::Transformation::Factory::TRANSFORMATION_FAILURE) {
	goto CLEANUP;
    }

    my $write_rpm_ok;
    if ($RPM::Source::TransformContainer::WRITER) {
	$write_rpm_ok=$RPM::Source::TransformContainer::WRITER->write($spec,$parent,$opt,$resourceobj);
    }
  CLEANUP:
    $parent->cleanup() if $parent;
    $spec->cleanup() if $write_rpm_ok || $opt->{-cleanup};
    
}

1;
